@@ -1,50 +1,17 @@
 import logging
-import re
+import time
 
-import six
-from six.moves import configparser
+import eventlet
 
-from ironicclient import client, exceptions
-from keystoneclient.v2_0 import client as keystone
+from ironicclient import exceptions
 
+from ironic_discoverd import conf
 from ironic_discoverd import firewall
+from ironic_discoverd import utils
 
 
 LOG = logging.getLogger("discoverd")
 ALLOW_SEARCH_BY_MAC = True
-OS_ARGS = ('os_password', 'os_username', 'os_auth_url', 'os_tenant_name')
-
-
-def init_conf():
-    global CONF
-    CONF = configparser.ConfigParser(
-        defaults={'debug': 'false',
-                  'listen_address': '0.0.0.0',
-                  'listen_port': '5050',
-                  'dnsmasq_interface': 'br-ctlplane',
-                  'authenticate': 'true',
-                  'firewall_update_period': '15',
-                  'ports_for_inactive_interfaces': 'false'})
-
-
-CONF = None
-init_conf()
-
-
-def get_client():
-    args = dict((k, CONF.get('discoverd', k)) for k in OS_ARGS)
-    return client.get_client(1, **args)
-
-
-def get_keystone(token):
-    return keystone.Client(token=token, auth_url=CONF.get('discoverd',
-                                                          'os_auth_url'))
-
-
-def is_valid_mac(address):
-    m = "[0-9a-f]{2}(:[0-9a-f]{2}){5}$"
-    return (isinstance(address, six.string_types)
-            and re.match(m, address.lower()))
 
 
 def process(node_info):
@@ -54,7 +21,7 @@ def process(node_info):
                   node_info['error'])
         return
 
-    compat = CONF.getboolean('discoverd', 'ports_for_inactive_interfaces')
+    compat = conf.getboolean('discoverd', 'ports_for_inactive_interfaces')
     if 'interfaces' not in node_info and 'macs' in node_info:
         LOG.warning('Using "macs" field is deprecated, please '
                     'update your discovery ramdisk')
@@ -78,7 +45,7 @@ def process(node_info):
 
     valid_interfaces = {
         n: iface for n, iface in node_info['interfaces'].items()
-        if is_valid_mac(iface['mac']) and (compat or iface.get('ip'))
+        if utils.is_valid_mac(iface['mac']) and (compat or iface.get('ip'))
     }
     valid_macs = [iface['mac'] for iface in valid_interfaces.values()]
     if valid_interfaces != node_info['interfaces']:
@@ -92,7 +59,7 @@ def process(node_info):
              'ipmi_address': node_info.get('ipmi_address')})
         LOG.info('Eligible interfaces are %s', valid_interfaces)
 
-    ironic = get_client()
+    ironic = utils.get_client()
     bmc_known = bool(node_info.get('ipmi_address'))
     if bmc_known:
         # TODO(dtantsur): bulk loading
@@ -170,31 +137,80 @@ def process(node_info):
                   'management configuration:\n%s', node.uuid, exc)
 
 
+class DiscoveryFailed(Exception):
+    def __init__(self, msg, code=400):
+        super(DiscoveryFailed, self).__init__(msg)
+        self.http_code = code
+
+
 def discover(uuids):
     """Initiate discovery for given node uuids."""
-    ironic = get_client()
+    if not uuids:
+        raise DiscoveryFailed("No nodes to discover")
+
+    ironic = utils.get_client()
     LOG.debug('Validating nodes %s', uuids)
     nodes = []
-    patch = [{'op': 'add', 'path': '/extra/on_discovery', 'value': 'true'},
-             {'op': 'replace', 'path': '/maintenance', 'value': 'true'}]
     for uuid in uuids:
         try:
             node = ironic.node.get(uuid)
-        except exceptions.HTTPClientError:
-            LOG.exception('Failed validation of node %s', uuid)
-            continue
+        except exceptions.NotFound:
+            LOG.error('Node %s cannot be found', uuid)
+            raise DiscoveryFailed("Cannot find node %s" % uuid, code=404)
+        except exceptions.HttpError as exc:
+            LOG.exception('Cannot get node %s', uuid)
+            raise DiscoveryFailed("Cannot get node %s: %s" % (uuid, exc))
 
-        if not node.maintenance:
-            LOG.warning('Node %s will be put in maintenance mode', node.uuid)
+        _validate(ironic, node)
+
+        if node.extra.get('on_discovery'):
+            LOG.warning('Node %s seems to be on discovery already', node.uuid)
 
-        ironic.node.update(uuid, patch)
         nodes.append(node)
 
-    if not nodes:
-        LOG.error('No nodes to discover')
-        return
-
     LOG.info('Proceeding with discovery on nodes %s', [n.uuid for n in nodes])
+    eventlet.greenthread.spawn_n(_background_discover, ironic, nodes)
+
+
+def _validate(ironic, node):
+    if node.instance_uuid:
+        LOG.error('Refusing to discover node %s with assigned instance_uuid',
+                  node.uuid)
+        raise DiscoveryFailed(
+            'Refusing to discover node %s with assigned instance uuid' %
+            node.uuid)
+
+    power_state = node.power_state
+    if (not node.maintenance and power_state is not None
+            and power_state.lower() != 'power off'):
+        LOG.error('Refusing to discover node %s with power_state "%s" '
+                  'and maintenance mode off',
+                  node.uuid, power_state)
+        raise DiscoveryFailed(
+            'Refusing to discover node %s with power state "%s" and '
+            'maintenance mode off' %
+            (node.uuid, power_state))
+
+    validation = ironic.node.validate(node.uuid)
+    if not validation.power['result']:
+        LOG.error('Failed validation of power interface for node %s, '
+                  'reason: %s', node.uuid, validation.power['reason'])
+        raise DiscoveryFailed('Failed validation of power interface for '
+                              'node %s' % node.uuid)
+
+
+def _background_discover(ironic, nodes):
+    patch = [{'op': 'add', 'path': '/extra/on_discovery', 'value': 'true'},
+             {'op': 'add', 'path': '/extra/discovery_timestamp',
+              'value': str(time.time())}]
+    for node in nodes:
+        node_patch = []
+        if not node.maintenance:
+            LOG.warning('Node %s will be put in maintenance mode', node.uuid)
+            node_patch.append(
+                {'op': 'replace', 'path': '/maintenance', 'value': 'true'})
+
+        ironic.node.update(node.uuid, patch + node_patch)
 
     to_exclude = set()
     for node in nodes:
@@ -209,7 +225,7 @@ def discover(uuids):
 
     for node in nodes:
         try:
-            ironic.node.set_power_state(node.uuid, 'on')
+            ironic.node.set_power_state(node.uuid, 'reboot')
         except Exception as exc:
             LOG.error('Failed to power on node %s, check it\'s power '
                       'management configuration:\n%s', node.uuid, exc)
